@@ -93,7 +93,9 @@ impl Sub<FileLen> for FilePos {
 
 /// Represents length of data, in bytes.
 /// Provides more type safety and nicer formatting over using a raw u64.
-#[derive(Clone, Copy, PartialEq, Eq, PartialOrd, Ord, Hash, Debug, Serialize, Default)]
+#[derive(
+    Clone, Copy, PartialEq, Eq, PartialOrd, Ord, Hash, Debug, Deserialize, Serialize, Default,
+)]
 pub struct FileLen(pub u64);
 
 impl FileLen {
@@ -257,7 +259,7 @@ impl FileInfo {
 
     #[cfg(target_os = "linux")]
     pub fn fetch_physical_location(&mut self) -> io::Result<u64> {
-        let new_location = get_physical_file_location(&self.path())?;
+        let new_location = get_physical_file_location(self.path())?;
         if let Some(new_location) = new_location {
             self.location = self.location & DEVICE_MASK | (new_location >> 8) & OFFSET_MASK;
         }
@@ -349,7 +351,7 @@ impl FileId {
 }
 
 pub(crate) fn file_id_or_log_err(file: &Path, log: &Log) -> Option<FileId> {
-    match FileId::new(&file) {
+    match FileId::new(file) {
         Ok(id) => Some(id),
         Err(e) if e.kind() == ErrorKind::NotFound => None,
         Err(e) => {
@@ -411,6 +413,17 @@ impl Serialize for FileHash {
     }
 }
 
+impl<'de> Deserialize<'de> for FileHash {
+    fn deserialize<D>(deserializer: D) -> Result<FileHash, D::Error>
+    where
+        D: Deserializer<'de>,
+    {
+        let s = String::deserialize(deserializer)?;
+        let hash_value = u128::from_str_radix(s.as_str(), 16).map_err(serde::de::Error::custom)?;
+        Ok(FileHash(hash_value))
+    }
+}
+
 #[cfg(unix)]
 fn to_off_t(offset: u64) -> libc::off_t {
     min(libc::off_t::MAX as u64, offset) as libc::off_t
@@ -475,7 +488,7 @@ fn evict_page_cache_if_low_mem(file: &mut File, len: FileLen) {
             let free_ratio = free_mem as f32 / total_mem as f32;
             if free_ratio < 0.05 {
                 evict_page_cache(
-                    &file,
+                    file,
                     FilePos::zero() + skipped_prefix_len,
                     len - skipped_prefix_len,
                 );
@@ -499,7 +512,7 @@ pub(crate) enum Caching {
 /// Opens a file and positions it at the given offset.
 /// Additionally, sends the advice to the operating system about how many bytes will be read.
 fn open(path: &Path, offset: FilePos, len: FileLen, cache_policy: Caching) -> io::Result<File> {
-    let mut file = open_noatime(&path)?;
+    let mut file = open_noatime(path)?;
     configure_readahead(&file, offset, len, cache_policy);
     if offset > FilePos::zero() {
         file.seek(offset.into())?;
@@ -628,14 +641,14 @@ pub(crate) fn file_hash_or_log_err(
 
 #[cfg(test)]
 mod test {
-
-    use super::*;
-
-    use crate::path::Path;
     use std::fs::{create_dir_all, File};
     use std::io::Write;
     use std::path::PathBuf;
 
+    use crate::path::Path;
+
+    use super::*;
+
     #[test]
     fn test_format_bytes() {
         let file_len = FileLen(16000);
